/*

*/
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#include "lowport.h"

//----------------------------------------------------------------------------------
// constant definitions
//----------------------------------------------------------------------------------
#define TIMER_1_CNT     0xf85f;  // 0.5 sec, use AVRcalc to calculate these values
                                 // (TCNT1H=0xf8, TCNT1L=0x5f)

/* this constants should be in <avr/io8515.h> !! */
#define TMC16_CK1024	(_BV(CS12) + _BV(CS10))


//----------------------------------------------------------------------------------
//  module global variables
//----------------------------------------------------------------------------------

static volatile unsigned int uiByteCount;     // use volatile when variable is accessed from interrupts

static volatile unsigned int uiTimeUpperLight;     // use volatile when variable is accessed from interrupts
static volatile unsigned int uiTimeDownLight;     // use volatile when variable is accessed from interrupts

//----------------------------------------------------------------------------------
// functions
//----------------------------------------------------------------------------------
void Init ()
  {
    uiTimeUpperLight = 300;   //Read the value from EEProm
    uiTimeDownLight  = 300;   //Read the value from EEProm
  }

//----------------------------------------------------------------------------------
// Interrupts
//----------------------------------------------------------------------------------
SIGNAL(SIG_OVERFLOW1)            // handler for tcnt1 overflow interrupt
  {
    uiByteCount++;
    if (uiByteCount%3 == 0)
      {
        uiTimeUpperLight--;
        uiTimeDownLight--;
        LowPortSetByte (1, uiTimeUpperLight);//0xFF ^ ucToBCD [uiByteCount]);
        LowPortSetByte (2, uiTimeDownLight);//0xFF ^ ucToBCD [uiByteCount]);
        if (uiTimeUpperLight == 0)
          {
            uiTimeUpperLight = 300;
          }
        if (uiTimeDownLight == 0)
          {
            uiTimeDownLight = 300;
          }
      }
    TCNT1 = TIMER_1_CNT;         // reset counter to get this interrupt again
  }

int main(void)
  {    
    unsigned int i, a, test, z;
    unsigned char v;
    
    
    DDRB  = 0xff;                // use all pins on PortB for output 
    DDRD  = 0xf0;                // use all pins on PortB for output 
    PORTB = 0;                  // and turn off all LEDs

    //DDRD  = 0xff;//0xe0;                // use this pins on PortD for output 
    //PORTD = 0xe0;                // and turn off all LEDs
    
    TCCR1A = 0x00;               // disable PWM and Compare Output Mode
    TCCR1B = TMC16_CK1024;       // use CLK/1024 prescale value
    
    TCNT1  = TIMER_1_CNT         // reset TCNT1 high/low byte

    TIMSK  = _BV(TOIE1);         // enable TCNT1 overflow 
    
    uiByteCount = 0;                     // init variable representing the LED state 

    sei();                       // enable interrupts 
    
    set_bit (PORTB, PB4); // /LE
    for (;;)                    //forever
      {
        clear_bit (PORTB, PB4); // /LE
        test = ucDisplayValue [3];
        clear_bit (PORTB, PB1);
        for (a = 0; a < 2; a++)
          {
            v = 1;
            for (i = 0; i < 4; i++)
              {
                
                if (i & 0x01 == 0x01)
                  set_bit (PORTB, PB2); //A0
                else
                  clear_bit (PORTB, PB2);
    
                if (i & 0x02 == 0x02)
                  set_bit (PORTB, PB3); //A0
                else
                  clear_bit (PORTB, PB3);

                if (test & v == v)
                  clear_bit (PORTB, PB0);
                else
                  set_bit (PORTB, PB0); //DATA
                
                v <<= i;
              }
              test = ucDisplayValue [3] >> 4;
              set_bit (PORTB, PB1);
          }
        for (z = 0; z < 1000; z++){};
      }                  
  }
